var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
 *
 * BIGTINCAN - CONFIDENTIAL
 *
 * All Rights Reserved.
 *
 * NOTICE: All information contained herein is, and remains the property of BigTinCan Mobile Pty Ltd and its suppliers,
 * if any. The intellectual and technical concepts contained herein are proprietary to BigTinCan Mobile Pty Ltd and its
 * suppliers and may be covered by U.S. and Foreign Patents, patents in process, and are protected by trade secret or
 * copyright law. Dissemination of this information or reproduction of this material is strictly forbidden unless prior
 * written permission is obtained from BigTinCan Mobile Pty Ltd.
 *
 * @package btca-client
 * @copyright 2010-2017 BigTinCan Mobile Pty Ltd
 * @author Lochlan McBride <lochlan.mcbride@bigtincan.com>
 */

var BTCAClient = function () {
  function BTCAClient(options) {
    _classCallCheck(this, BTCAClient);

    // Warn if handle is not set
    if (!options.handle) {
      console.warn('Please set a `handle` for your BTCA.');
    }

    // Log banner
    this.logBanner = '######### BTCJSBridge 3.0 Message #########:  \n';

    // API schema
    this.btcjsapiRequestSchema = 'btcjsapi://';

    // array for caching operations
    this.btcjsapiOperations = [];

    // cache request response data
    this.cache = false;

    // Prefix for anonymous callbacks
    this.callbackActionPrefix = 'BTCA_CB_';

    // Disable btcjsapi:// and mailto: actions
    this.disableEvents = false;

    // json object of listeners
    this.jsListeners = {};

    // Provide a handle on init to identify the BTCA
    this.handle = 'BTCAClient_' + this.generateUUID();

    // Print log messages
    this.log = false;

    // postMessage 'source' attribute for receiver to filter messages
    this.pmSource = 'btc-js-bridge';

    // parent window.location.origin for postMessage
    this.parentUrl = '*';

    // requestMethod (iframe, postMessage)
    this.requestMethod = 'postMessage';

    // Set passed options
    for (var key in options) {
      switch (key) {
        case 'handle':
          this.handle = options[key] + '_' + this.generateUUID();
          break;
        default:
          this[key] = options[key];
          break;
      }
    }

    this.init(); // initialise
  }

  /**
   * initialise the instance, create events and detect requestMethod
   */


  _createClass(BTCAClient, [{
    key: 'init',
    value: function init() {
      var _this = this;

      // Wait for load event if document is not ready
      if (window && window.document && window.document.readyState !== 'complete') {
        window.addEventListener('load', function () {
          _this.createEvents();
        });
      } else {
        this.createEvents();
      }

      /**
       * Create postMessage listener if loaded with required params
       * e.g. http://hostname.com/index.html?referrer=parentUrl&action=postMessage
       */
      this.params = this.parseQuery(window.location.search);
      if (this.params.referrer && this.params.action && this.params.action === 'postMessage') {
        this.requestMethod = 'postMessage';
        this.parentUrl = this.params.referrer;
      }
      if (this.parentUrl && this.requestMethod && this.requestMethod === 'postMessage') {
        this.createMessageListener();
      }

      this.logMessage(this.logBanner + 'requestMethod: ' + this.requestMethod);
    }
  }, {
    key: 'createEvents',


    /**
     * Attach events to anchor tags
     */
    value: function createEvents() {
      var _this2 = this;

      if (!this.disableEvents) {
        var anchors = document.body.getElementsByTagName('a');
        if (anchors.length) {
          Array.prototype.forEach.call(anchors, function (item) {
            var href = item.getAttribute('href');

            // Ignore anchors without href or hash
            if (!href || href.indexOf('#') === 0) {
              return false;
            }

            // btcjsapi:// links
            if (href.indexOf(_this2.btcjsapiRequestSchema) === 0) {
              item.addEventListener('click', function (event) {
                event.preventDefault();
                _this2.send(item.getAttribute('href'));
              });

              // mailto: links
            } else if (href.indexOf('mailto:') === 0) {
              item.addEventListener('click', function (event) {
                event.preventDefault();
                var href = item.getAttribute('href');
                var requestString = 'btcjsapi://sendEmail?';
                var params = _this2.parseMailtoUrl(href);
                var data = {
                  action: 'sendEmail',
                  params: params,
                  requestString: requestString + _this2.serialize(params.data)
                };
                _this2.makeRequest(data);
              });

              // all other anchors
            } else {
              item.addEventListener('click', function (event) {
                var href = item.getAttribute('href');
                var target = item.getAttribute('target');
                var title = item.getAttribute('title');
                var requestString = 'btcjsapi://openURL?url=' + href;
                var rel = item.getAttribute('rel');

                // Treat links starting with absolute url as app
                if (!rel && href.indexOf('/') === 0) {
                  rel = 'app';

                  // Treat links starting with http as external
                } else if (!rel && href.indexOf('http') === 0) {
                  rel = 'external';
                }

                // Prevent default for app and external links
                if (rel === 'app' || rel === 'external' || target) {
                  event.preventDefault();
                }

                var data = {
                  action: 'openURL',
                  params: {
                    url: href,
                    rel: rel,
                    target: target,
                    title: title
                  },
                  requestString: requestString
                };
                _this2.makeRequest(data);
              });
            }
          });
        }
      }
    }
  }, {
    key: 'btcDebugger',


    /**
     * btcDebugger is a bug reporter, when an error is returned from the bridge,
     * it will handle the error. It is available globally as `BTCJSAPIDebugger`
     *
     * @param data  un-parsed result data from the bridge
     */
    value: function btcDebugger(data) {
      var json = this.parseJsonData(data);
      if (json !== null && json.error.message) {
        this.logMessage(this.logBanner + 'debug error message: ' + json.error.message);
      }
    }
  }, {
    key: 'requestTriggerBeforeFunction',


    /**
     * Default request trigger before function if any
     */
    value: function requestTriggerBeforeFunction(requestString) {
      this.logMessage(this.logBanner + 'trigger before requestString: ' + requestString);
    }
  }, {
    key: 'defaultJSListener',


    /**
     * Default js listener, which will be called anyway when any response from the request
     */
    value: function defaultJSListener(data, result, requestId, error) {
      this.logMessage(this.logBanner + 'response from request: ' + JSON.stringify(data, null, '  '));
    }
  }, {
    key: 'createMessageListener',


    /**
     * Listen for window message events
     */
    value: function createMessageListener() {
      var _this3 = this;

      window.addEventListener('message', function (e) {
        _this3.handlePostMessage(e.data, e);
      });
    }
  }, {
    key: 'logMessage',


    /**
     * Console.log message when log enabled
     */
    value: function logMessage(_logMessage) {
      if (this.log === true) {
        console.log(_logMessage);
      }
    }
  }, {
    key: 'parseJsonData',


    /**
     * Helper method to parse json data
     * @return parsed json object or null if fail
     */
    value: function parseJsonData(data) {
      if (data) {
        try {
          var result = JSON.parse(data);
          return result;
        } catch (err) {
          // Do something about the exception here
          this.logMessage(this.logBanner + 'json error on parsing data: ' + data);
          console.log(data);
          return null;
        }
      } else {
        return null;
      }
    }
  }, {
    key: 'unescapeHtml',


    /**
     * unescape html helper
     * @param escapedHtmlString
     * @return unescaped data
     */
    value: function unescapeHtml(string) {
      if (typeof DOMParser !== 'function') {
        var e = document.createElement('div');
        e.innerHTML = string;
        return e.childNodes.length === 0 ? '' : e.childNodes[0].nodeValue;
      }

      var doc = new DOMParser().parseFromString(string, 'text/html');
      return doc.documentElement.textContent;
    }
  }, {
    key: 'escapeHtml',


    /**
     * escape html helper
     * @param unescapedHtmlString
     * @return escaped string
     */
    value: function escapeHtml(string) {
      var map = {
        '&': '&amp;',
        '<': '&lt;',
        '>': '&gt;',
        '"': '&quot;',
        "'": '&#039;'
      };

      return string.replace(/[&<>"']/g, function (m) {
        return map[m];
      });
    }
  }, {
    key: 'parseApiUrl',


    /**
     * Parse btcjsapi:// URL to object
     *
     * @param {String} href
     * @return {Object} bridge params
     */
    value: function parseApiUrl(href) {
      var requestString = this.unescapeHtml(href);
      var path = requestString.split(this.btcjsapiRequestSchema)[1].split('?');
      var pathAction = path[0];
      var pathArgs = path[1];
      var pathData = this.parseQuery(pathArgs);
      var jsListener = null;

      if (pathData && pathData.jsListener) {
        jsListener = pathData.jsListener;

        // Remove jsListener from pathData
        delete pathData.jsListener;
      }

      return {
        action: pathAction,
        params: pathData,
        jsListener: jsListener
      };
    }
  }, {
    key: 'parseMailtoUrl',


    /**
     * Parse a mailto: URL to object
     *
     * @param {String} href
     * @return {Object} sendMail data params
     */
    value: function parseMailtoUrl(href) {
      var params = {};
      var to = href.split('mailto:')[1];
      var cc = '';
      var bcc = '';
      var subject = '';

      // mailto: link has params
      if (to.indexOf('?') > 10) {
        var splitString = to.split('?');
        recipients = splitString[0];
        params = this.parseQuery(splitString[1]);

        // Remove white space and split to array
        to = recipients.replace(/\s/g, '').split(',');
        cc = params.cc.replace(/\s/g, '').split(',');
        bcc = params.bcc.replace(/\s/g, '').split(',');
      }

      return {
        to: to,
        cc: cc,
        bcc: bcc,
        subject: params.subject || '',
        body: params.body || ''
      };
    }

    /**
     * Convert query string to JSON object
     *
     * @param {String} qstr
     * @returns {Object}
     */

  }, {
    key: 'parseQuery',
    value: function parseQuery(qstr) {
      var query = {};
      if (!qstr) {
        return query;
      }
      var a = (qstr[0] === '?' ? qstr.substr(1) : qstr).split('&');
      for (var i = 0; i < a.length; i++) {
        var b = a[i].split('=');
        query[decodeURIComponent(b[0])] = decodeURIComponent(b[1] || '');
      }
      return query;
    }
  }, {
    key: 'serialize',


    /**
     * Convert JSON object to query string
     */
    value: function serialize(obj, prefix) {
      var str = [];
      var p = void 0;
      for (p in obj) {
        if (obj.hasOwnProperty(p)) {
          var k = prefix ? prefix + '[' + p + ']' : p,
              v = obj[p];
          str.push(v !== null && (typeof v === 'undefined' ? 'undefined' : _typeof(v)) === 'object' ? this.serialize(v, k) : encodeURIComponent(k) + '=' + encodeURIComponent(v));
        }
      }
      return str.join('&');
    }
  }, {
    key: 'generateUUID',


    /**
     * Fast UUID generator, RFC4122 version 4 compliant.
     * http://stackoverflow.com/questions/105034/how-to-create-a-guid-uuid-in-javascript/21963136#21963136
     */
    value: function generateUUID() {
      var lut = [];for (var i = 0; i < 256; i++) {
        lut[i] = (i < 16 ? '0' : '') + i.toString(16);
      }
      var d0 = Math.random() * 0xffffffff | 0;
      var d1 = Math.random() * 0xffffffff | 0;
      var d2 = Math.random() * 0xffffffff | 0;
      var d3 = Math.random() * 0xffffffff | 0;
      return lut[d0 & 0xff] + lut[d0 >> 8 & 0xff] + lut[d0 >> 16 & 0xff] + lut[d0 >> 24 & 0xff] + '-' + lut[d1 & 0xff] + lut[d1 >> 8 & 0xff] + '-' + lut[d1 >> 16 & 0x0f | 0x40] + lut[d1 >> 24 & 0xff] + '-' + lut[d2 & 0x3f | 0x80] + lut[d2 >> 8 & 0xff] + '-' + lut[d2 >> 16 & 0xff] + lut[d2 >> 24 & 0xff] + lut[d3 & 0xff] + lut[d3 >> 8 & 0xff] + lut[d3 >> 16 & 0xff] + lut[d3 >> 24 & 0xff];
    }
  }, {
    key: 'makeRequest',


    /**
     * Makes request using the selected requestMethod
     *
     * @param {any} requestData valid bridge data
     */
    value: function makeRequest(requestData) {
      this.requestTriggerBeforeFunction(requestData.requestString);
      var finalRequestData = this.finalRequestPreparation(requestData);

      // postMessage request
      if (this.requestMethod === 'postMessage') {
        this.postMessageRequest(finalRequestData);

        // iframe is default method
      } else {
        this.iframeRequestCall(finalRequestData.requestString);
      }
    }
  }, {
    key: 'finalRequestPreparation',


    /**
     * Final request preparation
     * generate the requestId and cache the request
     *
     * @param requestData
     * @return finalRequestData
     */
    value: function finalRequestPreparation(requestData) {
      var symbol = this.connectionQuerySymbol(requestData.requestString);
      var requestId = this.btcjsapiOperations.length;
      var finalRequestData = requestData;

      finalRequestData.requestString += symbol + 'requestId=' + requestId;
      finalRequestData.requestId = requestId;

      // Cache request
      var request = {
        requestData: finalRequestData,
        responseData: ''
      };
      this.btcjsapiOperations.push(request);

      return finalRequestData;
    }
  }, {
    key: 'connectionQuerySymbol',
    value: function connectionQuerySymbol(requestString) {
      var hasQueryInRequestString = requestString.indexOf('?') > -1;
      var symbol = hasQueryInRequestString ? '&' : '?';
      return symbol;
    }
  }, {
    key: 'iframeRequestCall',


    /**
     * Send request using temporary iframe, devices intercept this request
     *
     * @param {String} requestString
     */
    value: function iframeRequestCall(requestString) {
      var finalStringWithCallback = requestString.replace('jsListener', 'originalJsListener');
      finalStringWithCallback += '&jsListener=responseFromRequest';
      var apiRequestString = this.btcjsapiRequestSchema + finalStringWithCallback;

      this.logMessage(this.logBanner + 'request making: ' + apiRequestString);

      // Create temporary iframe with src as requestString
      var iframe = document.createElement('iframe');
      iframe.setAttribute('src', apiRequestString);
      document.documentElement.appendChild(iframe);
      iframe.parentNode.removeChild(iframe);
      iframe = null;
    }
  }, {
    key: 'postMessageRequest',


    /**
     * Send requestData using postMessage
     *
     * @param {Object} requestData
     */
    value: function postMessageRequest(requestData) {
      var data = {
        action: 'request',
        data: requestData,
        handle: this.handle,
        parentUrl: this.parentUrl,
        source: this.pmSource
      };

      this.logMessage(this.logBanner + 'postMessage: ' + JSON.stringify(data, null, '  '));

      // post message to parent frame
      try {
        window.parent.postMessage(data, this.parentUrl);
      } catch (error) {
        // return error response to callback
        var errorData = {
          result: null,
          error: {
            code: 99,
            message: 'Not in iframe'
          },
          originalRequest: data.data
        };
        this.responseFromRequest(errorData);
      }
    }
  }, {
    key: 'handlePostMessage',


    /**
     * Handle postMessage event
     *
     * @param {Object} data
     * @param {Object} event
     */
    value: function handlePostMessage(data, event) {
      if (!data) {
        return false;
      }

      // Parse string to json
      var json = data;
      if (json && typeof json === 'string') {
        json = this.parseJsonData(json);
      }

      // make sure we are dealing with a js-bridge response
      if (!json.originalRequest || json.originalRequest.source !== this.pmSource) {
        return false;
      }

      // Pass to response handler
      this.responseFromRequest(json);
    }
  }, {
    key: 'responseFromRequest',


    /**
     * Handle response from the request
     *
     * @param {Object} data   un-parsed result data from the bridge
     */
    value: function responseFromRequest(json) {
      // log the response
      this.logMessage(this.logBanner + 'got response: ' + JSON.stringify(json, null, '  '));

      var requestId = json.originalRequest.data.requestId;
      var jsListener = json.originalRequest.data.jsListener;
      var error = json.error;
      var result = json.result;
      var fn = this.jsListeners[jsListener];
      var operation = this.btcjsapiOperations[requestId];

      if (typeof operation !== 'undefined') {
        if (this.cache) {
          operation.responseData = json;
          this.btcjsapiOperations[requestId] = operation; // overwrite it
        }
      }

      // Execute callback
      if (fn && typeof fn === 'function') {
        fn(result, requestId, error);
      }

      // Remove generated callback
      if (jsListener && jsListener.indexOf(this.callbackActionPrefix) === 0) {
        delete this.jsListeners[jsListener];
      }

      // Trigger defaultJSListener
      this.defaultJSListener(json, result, requestId, error);
    }
  }, {
    key: 'send',


    /**
     * Accepts valid js-bridge data or btcjsapi:// string
     * and calls makeRequest()
     */
    value: function send(data, callback) {
      var cbName = void 0;
      var json = data;
      var requestString = '';

      // No data
      if (!data) {
        throw new Error('`data` must be passed as first parameter to send()');
      }

      // Data is a string
      if (typeof data === 'string') {
        // Parse btcjsapi:// string
        if (data.indexOf(this.btcjsapiRequestSchema) === 0) {
          requestString = data.replace(this.btcjsapiRequestSchema, '');
          json = this.parseApiUrl(json);

          // Handle non-api string
        } else {
          throw new Error('Invalid request string string provided');
        }
      }

      // No action passed
      if (!json.action) {
        throw new Error('`data` must have an `action`');
      }

      // Passed 'callback' takes priority over data.jsListener
      if (callback) {
        // Anonymous callback
        if (typeof callback === 'function' && !callback.name) {
          // Generate function name
          cbName = this.callbackActionPrefix + this.generateUUID();

          // Named callback
        } else {
          cbName = typeof callback === 'function' ? callback.name : callback;
        }

        // Add listener
        json.jsListener = cbName;
        this.jsListeners[json.jsListener] = callback;

        // global jsListener passed in data
      } else if (json.jsListener && window[json.jsListener]) {
        this.jsListeners[json.jsListener] = window[json.jsListener];
      }

      // No callback/jsListener
      if (typeof this.jsListeners[json.jsListener] !== 'function') {
        throw new Error('Invalid callback/jsListener defined. Must be a function or function reference.');
      }

      // Parse object to btcjsapi:// string
      if (!requestString) {
        var dataString = this.serialize(json.param);
        requestString = json.action;

        if (dataString) {
          requestString += '?' + dataString;
        }

        if (json.jsListener) {
          requestString += (dataString ? '&' : '?') + 'jsListener=' + (json.jsListener.name || json.jsListener);
        }
      }

      // Create API data
      var apiData = {
        action: json.action,
        params: json.params,
        jsListener: json.jsListener ? json.jsListener.name || json.jsListener : null,
        requestString: requestString
      };

      this.makeRequest(apiData);
    }
  }, {
    key: 'getRequest',


    /**
     * Get cached request by ID
     *
     * @param {Number} requestId
     * @returns reqest
     */
    value: function getRequest(requestId) {
      if ((typeof requestId === 'undefined' ? 'undefined' : _typeof(requestId)) === undefined) {
        return this.btcjsapiOperations;
      }

      var request = this.btcjsapiOperations[requestId];
      if (!request) {
        this.logMessage('No history found, please make sure cache is enabled.');
      }
      return request;
    }
  }]);

  return BTCAClient;
}();

;

// Universal Module Definition
// Set up AMD and CommonJS export
// https://github.com/umdjs/umd
(function (root, factory) {
  if (typeof define === 'function' && define.amd) {
    // AMD. Register as an anonymous module.
    define([], factory);
  } else if ((typeof module === 'undefined' ? 'undefined' : _typeof(module)) === 'object' && module.exports) {
    // Node. Does not work with strict CommonJS
    module.exports = factory();
  } else {
    // Browser globals (root is window)
    root.returnExports = factory();
  }
})(this, function () {
  return BTCAClient;
});